@@ -68,10 +68,18 @@ struct _PeasObjectModulePrivate {
   gchar *path;
   gchar *module_name;
 
-  gboolean types_registered : 1;
-  gboolean resident : 1;
+  guint resident : 1;
 };
 
+static void
+peas_object_module_register_types (PeasObjectModule *module)
+{
+  g_return_if_fail (PEAS_IS_OBJECT_MODULE (module));
+  g_return_if_fail (module->priv->register_func != NULL);
+
+  module->priv->register_func (module);
+}
+
 static gboolean
 peas_object_module_load (GTypeModule *gmodule)
 {
@@ -124,6 +132,8 @@ peas_object_module_load (GTypeModule *gmodule)
   if (module->priv->resident)
     g_module_make_resident (module->priv->library);
 
+  peas_object_module_register_types (module);
+
   return TRUE;
 }
 
@@ -275,17 +285,6 @@ peas_object_module_new (const gchar *module_name,
                                            NULL));
 }
 
-void
-peas_object_module_register_types (PeasObjectModule *module)
-{
-  g_return_if_fail (PEAS_IS_OBJECT_MODULE (module));
-  g_return_if_fail (module->priv->register_func != NULL);
-  g_return_if_fail (module->priv->types_registered == FALSE);
-
-  module->priv->register_func (module);
-  module->priv->types_registered = TRUE;
-}
-
 GObject *
 peas_object_module_create_object (PeasObjectModule *module,
                                   GType             interface,
@@ -297,9 +296,6 @@ peas_object_module_create_object (PeasObjectModule *module,
 
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), NULL);
 
-  if (!module->priv->types_registered)
-    peas_object_module_register_types (module);
-
   impls = (InterfaceImplementation *) module->priv->implementations->data;
   for (i = 0; i < module->priv->implementations->len; ++i)
     if (impls[i].iface_type == interface)
@@ -317,9 +313,6 @@ peas_object_module_provides_object (PeasObjectModule *module,
 
   g_return_val_if_fail (PEAS_IS_OBJECT_MODULE (module), FALSE);
 
-  if (!module->priv->types_registered)
-    peas_object_module_register_types (module);
-
   impls = (InterfaceImplementation *) module->priv->implementations->data;
   for (i = 0; i < module->priv->implementations->len; ++i)
     if (impls[i].iface_type == interface)
@@ -366,7 +359,7 @@ peas_object_module_get_library (PeasObjectModule *module)
  * requested.
  *
  * This method is primarily meant to be used by native bindings (like gtkmm),
- * creatint native types which cannot be instantiated correctly using
+ * creating native types which cannot be instantiated correctly using
  * g_object_new().  For other uses, you will usually prefer relying on
  * peas_object_module_register_extension_type().
  */
@@ -402,7 +395,7 @@ create_gobject_from_type (guint       n_parameters,
    * property to avoid a warning. */
   if (n_parameters > 1 &&
       strcmp (parameters[n_parameters-1].name, "plugin-info") == 0 &&
-      g_object_class_find_property (cls, "plugin-info") != NULL)
+      g_object_class_find_property (cls, "plugin-info") == NULL)
     n_parameters --;
 
   instance = G_OBJECT (g_object_newv (exten_type, n_parameters, parameters));
