@@ -30,11 +30,33 @@
 #include "peas-i18n.h"
 #include "peas-plugin-info-priv.h"
 
+#ifdef G_OS_WIN32
+#define OS_HELP_KEY "Help-Windows"
+#elif defined(OS_OSX)
+#define OS_HELP_KEY "Help-MacOS-X"
+#else
+#define OS_HELP_KEY "Help-GNOME"
+#endif
+
 /**
  * SECTION:peas-plugin-info
  * @short_description: Information about a plugin.
  *
  * A #PeasPluginInfo contains all the information available about a plugin.
+ *
+ * All this information comes from the related plugin info file. Here is an
+ * example of such a plugin file, in the #GKeyFile format:
+ * |[
+ * [YourApplication Plugin]
+ * Module=helloworld
+ * Name=Hello World
+ * Description=Displays "Hello World"
+ * Authors=Steve Frécinaux &lt;code@istique.net&gt;
+ * Copyright=Copyright © 2009-10 Steve Frécinaux
+ * Website=http://live.gnome.org/Libpeas
+ * Help=http://library.gnome.org/devel/libpeas/unstable/
+ * IAge=2
+ * ]|
  **/
 
 PeasPluginInfo *
@@ -64,19 +86,12 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   g_free (info->copyright);
   g_free (info->loader);
   g_free (info->version);
+  g_free (info->help_uri);
   g_strfreev (info->authors);
 
   g_free (info);
 }
 
-/*
- * peas_plugin_info_get_type:
- *
- * Retrieves the #GType object which is associated with the #PeasPluginInfo
- * class.
- *
- * Return value: the GType associated with #PeasPluginInfo.
- **/
 GType
 peas_plugin_info_get_type (void)
 {
@@ -123,7 +138,9 @@ parse_extra_keys (PeasPluginInfo   *info,
           g_str_equal (keys[i], "Authors") ||
           g_str_equal (keys[i], "Copyright") ||
           g_str_equal (keys[i], "Website") ||
-          g_str_equal (keys[i], "Version"))
+          g_str_equal (keys[i], "Version") ||
+          g_str_has_prefix (keys[i], "Help") ||
+          g_str_equal (keys[i], "Builtin"))
         continue;
 
       b = g_key_file_get_boolean (plugin_file, section_header, keys[i], &error);
@@ -184,13 +201,14 @@ _peas_plugin_info_new (const gchar *filename,
   gchar *str;
   gchar **keys;
   gint integer;
+  gboolean b;
+  GError *error = NULL;
 
   g_return_val_if_fail (filename != NULL, NULL);
   g_return_val_if_fail (app_name != NULL, NULL);
 
   info = g_new0 (PeasPluginInfo, 1);
   info->refcount = 1;
-  info->visible = TRUE;
   info->file = g_strdup (filename);
 
   section_header = g_strdup_printf ("%s Plugin", app_name);
@@ -286,6 +304,24 @@ _peas_plugin_info_new (const gchar *filename,
   if (str)
     info->version = str;
 
+  /* Get Help URI */
+  str = g_key_file_get_string (plugin_file, section_header, OS_HELP_KEY, NULL);
+  if (str)
+    info->help_uri = str;
+  else
+    {
+      str = g_key_file_get_string (plugin_file, section_header, "Help", NULL);
+      if (str)
+        info->help_uri = str;
+    }
+
+  /* Get Builtin */
+  b = g_key_file_get_boolean (plugin_file, section_header, "Builtin", &error);
+  if (error != NULL)
+    g_clear_error (&error);
+  else
+    info->builtin = b;
+
   /* Get extra keys */
   keys = g_key_file_get_keys (plugin_file, section_header, NULL, NULL);
   parse_extra_keys (info, plugin_file, section_header, (const gchar **) keys);
@@ -335,9 +371,10 @@ peas_plugin_info_is_loaded (const PeasPluginInfo *info)
  * peas_plugin_info_is_available:
  * @info: A #PeasPluginInfo.
  *
- * Check if the plugin is available.  A plugin is marked as not available when
- * there is no loader available to load it, or when there has been an error
- * when trying to load it previously.
+ * Check if the plugin is available.
+ *
+ * A plugin is marked as not available when there is no loader available to
+ * load it, or when there has been an error when trying to load it previously.
  *
  * Returns: %TRUE if the plugin is available.
  */
@@ -349,12 +386,43 @@ peas_plugin_info_is_available (const PeasPluginInfo *info)
   return info->available != FALSE;
 }
 
+/**
+ * peas_plugin_info_is_builtin:
+ * @info: A #PeasPluginInfo.
+ *
+ * Gets is the plugin is a builtin plugin.
+ *
+ * A builtin plugin is a plugin which cannot be enabled or disabled by the
+ * user through a plugin manager (like #PeasUIPluginManager). Loading or
+ * unloading such plugins is the responsibility of the application alone.
+ * Most applications will usually load those plugins immediately after
+ * the initialization of the #PeasEngine.
+ *
+ * The relevant key in the plugin info file is "Builtin".
+ *
+ * Returns: %TRUE if the plugin is a builtin plugin, %FALSE
+ * if not.
+ **/
+gboolean
+peas_plugin_info_is_builtin (const PeasPluginInfo *info)
+{
+  g_return_val_if_fail (info != NULL, TRUE);
+
+  return info->builtin;
+}
+
 /**
  * peas_plugin_info_get_module_name:
  * @info: A #PeasPluginInfo.
  *
  * Gets the module name.
  *
+ * The module name will be used to find the actual plugin. The way this value
+ * will be used depends on the loader (i.e. on the language) of the plugin.
+ * This value is also used to uniquely identify a particular plugin.
+ *
+ * The relevant key in the plugin info file is "Module".
+ *
  * Returns: the module name.
  */
 const gchar *
@@ -371,6 +439,10 @@ peas_plugin_info_get_module_name (const PeasPluginInfo *info)
  *
  * Gets the module directory.
  *
+ * The module directory is the directory where the plugin file was found. This
+ * is not a value from the #GKeyFile, but rather a value provided by the
+ * #PeasEngine.
+ *
  * Returns: the module directory.
  */
 const gchar *
@@ -387,6 +459,11 @@ peas_plugin_info_get_module_dir (const PeasPluginInfo *info)
  *
  * Gets the data dir of the plugin.
  *
+ * The module data directory is the directory where a plugin should find its
+ * runtime data. This is not a value read from the #GKeyFile, but rather a
+ * value provided by the #PeasEngine, depending on where the plugin file was
+ * found.
+ *
  * Returns: the plugin's data dir.
  */
 const gchar *
@@ -397,12 +474,65 @@ peas_plugin_info_get_data_dir (const PeasPluginInfo *info)
   return info->data_dir;
 }
 
+/**
+ * peas_plugin_info_get_dependencies:
+ * @info: A #PeasPluginInfo.
+ *
+ * Gets the dependencies of the plugin.
+ *
+ * The #PeasEngine will always ensure that the dependencies of a plugin are
+ * loaded when the said plugin is loaded. It means that dependencies are
+ * loaded before the plugin, and unloaded after it. Circular dependencies of
+ * plugins lead to undefined loading order.
+ *
+ * The relevant key in the plugin info file is "Depends".
+ *
+ * Returns: the plugin's dependencies.
+ */
+const gchar **
+peas_plugin_info_get_dependencies (const PeasPluginInfo *info)
+{
+  g_return_val_if_fail (info != NULL, NULL);
+
+  return (const gchar **) info->dependencies;
+}
+
+/**
+ * peas_plugin_info_has_dependency:
+ * @info: A #PeasPluginInfo.
+ * @module_name: The name of the plugin to check.
+ *
+ * Check if the plugin depends on another plugin.
+ *
+ * Returns: whether the plugin depends on the plugin @module_name.
+ */
+gboolean
+peas_plugin_info_has_dependency (const PeasPluginInfo *info,
+                                 const gchar          *module_name)
+{
+  guint i;
+
+  g_return_val_if_fail (info != NULL, FALSE);
+  g_return_val_if_fail (module_name != NULL, FALSE);
+
+  for (i = 0; info->dependencies[i] != NULL; i++)
+    if (g_ascii_strcasecmp (module_name, info->dependencies[i]) == 0)
+      return TRUE;
+
+  return FALSE;
+}
+
+
 /**
  * peas_plugin_info_get_name:
  * @info: A #PeasPluginInfo.
  *
  * Gets the name of the plugin.
  *
+ * The name of a plugin should be a nice short string to be presented in UIs.
+ *
+ * The relevant key in the plugin info file is "Name".
+ *
  * Returns: the plugin's name.
  */
 const gchar *
@@ -419,6 +549,11 @@ peas_plugin_info_get_name (const PeasPluginInfo *info)
  *
  * Gets the description of the plugin.
  *
+ * The description of the plugin should be a string presenting the purpose of
+ * the plugin. It will typically be presented in a plugin's about box.
+ *
+ * The relevant key in the plugin info file is "Description".
+ *
  * Returns: the plugin's description.
  */
 const gchar *
@@ -429,12 +564,40 @@ peas_plugin_info_get_description (const PeasPluginInfo *info)
   return info->desc;
 }
 
+/**
+ * peas_plugin_info_get_icon_name:
+ * @info: A #PeasPluginInfo.
+ *
+ * Gets the icon name of the plugin.
+ *
+ * The icon of the plugin will be presented in the plugin manager UI. If no
+ * icon is specified, the default green puzzle icon will be used.
+ *
+ * The relevant key in the plugin info file is "Icon".
+ *
+ * Returns: the plugin's icon name.
+ */
+const gchar *
+peas_plugin_info_get_icon_name (const PeasPluginInfo *info)
+{
+  g_return_val_if_fail (info != NULL, NULL);
+
+  /* use the libpeas-plugin icon as a default if the plugin does not
+     have its own */
+  if (info->icon_name != NULL)
+    return info->icon_name;
+  else
+    return "libpeas-plugin";
+}
+
 /**
  * peas_plugin_info_get_authors:
  * @info: A #PeasPluginInfo.
  *
  * Gets a NULL-terminated array of strings with the authors of the plugin.
  *
+ * The relevant key in the plugin info file is "Authors".
+ *
  * Returns: the plugin's author list.
  */
 const gchar **
@@ -451,6 +614,8 @@ peas_plugin_info_get_authors (const PeasPluginInfo *info)
  *
  * Gets the website of the plugin.
  *
+ * The relevant key in the plugin info file is "Website".
+ *
  * Returns: the plugin's associated website.
  */
 const gchar *
@@ -467,6 +632,8 @@ peas_plugin_info_get_website (const PeasPluginInfo *info)
  *
  * Gets the copyright of the plugin.
  *
+ * The relevant key in the plugin info file is "Copyright".
+ *
  * Returns: the plugin's copyright information.
  */
 const gchar *
@@ -483,6 +650,8 @@ peas_plugin_info_get_copyright (const PeasPluginInfo *info)
  *
  * Gets the version of the plugin.
  *
+ * The relevant key in the plugin info file is "Version".
+ *
  * Returns: the plugin's version.
  */
 const gchar *
@@ -493,12 +662,38 @@ peas_plugin_info_get_version (const PeasPluginInfo *info)
   return info->version;
 }
 
+/**
+ * peas_plugin_info_get_help_uri:
+ * @info: A #PeasPluginInfo.
+ *
+ * Gets the help URI of the plugin.
+ *
+ * The Help URI of a plugin will typically be presented by the plugin manager
+ * as a "Help" button linking to the URI. It can either be a HTTP URL on some
+ * website or a ghelp: URI if a Gnome help page is available for the plugin.
+ *
+ * The relevant key in the plugin info file is "Help". Other platform-specific
+ * keys exist for platform-specific help files. Those are "Help-Gnome",
+ * "Help-Windows" and "Help-MacOs-X".
+ *
+ * Returns: the plugin's help URI.
+ */
+const gchar *
+peas_plugin_info_get_help_uri (const PeasPluginInfo *info)
+{
+  g_return_val_if_fail (info != NULL, NULL);
+
+  return info->help_uri;
+}
+
 /**
  * peas_plugin_info_get_iage:
  * @info: A #PeasPluginInfo.
  *
  * Gets the interface age of the plugin.
  *
+ * The relevant key in the plugin info file is "IAge".
+ *
  * Returns: the interface age of the plugin or %0 if not known.
  **/
 gint
@@ -516,8 +711,8 @@ peas_plugin_info_get_iage (const PeasPluginInfo *info)
  * Gets a hash table of string keys present and #GValue values,
  * present in the plugin information file, but not handled
  * by libpeas. Note that libpeas only handles booleans and
- * strings, and that strings that are recognised as booleans,
- * as done by #g_key_file_get_boolean, will be of boolean type.
+ * strings, and that strings that are recognized as booleans,
+ * as done by g_key_file_get_boolean(), will be of boolean type.
  *
  * Returns: a #GHashTable of string keys and #GValue values. Do
  * not free or destroy any data in this hashtable.
@@ -529,38 +724,3 @@ peas_plugin_info_get_keys (const PeasPluginInfo *info)
 
   return info->keys;
 }
-
-/**
- * peas_plugin_info_set_visible:
- * @info: A #PeasPluginInfo.
- * @visible: visibility of the plugin
- *
- * Sets whether the plugin should be visible in the
- * plugin manager.
- *
- **/
-void
-peas_plugin_info_set_visible (PeasPluginInfo *info,
-                              gboolean        visible)
-{
-  g_return_if_fail (info != NULL);
-
-  info->visible = visible;
-}
-
-/**
- * peas_plugin_info_get_visible:
- * @info: A #PeasPluginInfo.
- *
- * Gets the visibility of the plugin.
- *
- * Returns: %TRUE if the plugin should be visible, %FALSE
- * if not.
- **/
-gboolean
-peas_plugin_info_get_visible (const PeasPluginInfo *info)
-{
-  g_return_val_if_fail (info != NULL, TRUE);
-
-  return info->visible;
-}
